@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
    Samba utility functions
    Copyright (C) Andrew Tridgell 1992-1998
@@ -13,18 +13,19 @@
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #include "replace.h"
 #include <talloc.h>
+#include <tevent.h>
 #include "system/network.h"
 #include "system/filesys.h"
 #include "system/locale.h"
@@ -36,6 +37,7 @@
 #include "samba_util.h"
 #include "lib/util/select.h"
 #include <libgen.h>
+#include <gnutls/gnutls.h>
 
 #ifdef HAVE_SYS_PRCTL_H
 #include <sys/prctl.h>
@@ -114,17 +116,24 @@ _PUBLIC_ bool file_exist(const char *fname)
 }
 
 /**
- Check a files mod time.
-**/
-
-_PUBLIC_ time_t file_modtime(const char *fname)
+ * @brief Return a files modification time.
+ *
+ * @param fname  The name of the file.
+ *
+ * @param mt     A pointer to store the modification time.
+ *
+ * @return 0 on success, errno otherwise.
+ */
+_PUBLIC_ int file_modtime(const char *fname, struct timespec *mt)
 {
-	struct stat st;
-  
-	if (stat(fname,&st) != 0) 
-		return(0);
+	struct stat st = {0};
 
-	return(st.st_mtime);
+	if (stat(fname, &st) != 0) {
+		return errno;
+	}
+
+	*mt = get_mtimespec(&st);
+	return 0;
 }
 
 /**
@@ -419,9 +428,9 @@ _PUBLIC_ bool fcntl_lock(int fd, int op, off_t offset, off_t count, int type)
 	/* a lock query */
 	if (op == F_GETLK) {
 		if ((ret != -1) &&
-				(lock.l_type != F_UNLCK) && 
-				(lock.l_pid != 0) && 
-				(lock.l_pid != getpid())) {
+				(lock.l_type != F_UNLCK) &&
+				(lock.l_pid != 0) &&
+				(lock.l_pid != tevent_cached_getpid())) {
 			DEBUG(3,("fcntl_lock: fd %d is locked by pid %d\n",fd,(int)lock.l_pid));
 			return true;
 		}
@@ -486,10 +495,12 @@ static void dump_data_block16(const char *prefix, size_t idx,
 			      void (*cb)(const char *buf, void *private_data),
 			      void *private_data)
 {
-	char tmp[16];
+	size_t prefix_len = strlen(prefix);
+	/* 16 (=%04zX) + 2 (=[]) + 1 (='\0') => 19 */
+	char tmp[prefix_len + 19];
 	size_t i;
 
-	SMB_ASSERT(len >= 0 && len <= 16);
+	SMB_ASSERT(len <= 16);
 
 	snprintf(tmp, sizeof(tmp), "%s[%04zX]", prefix, idx);
 	cb(tmp, private_data);
@@ -805,7 +816,7 @@ _PUBLIC_ void *smb_memdup(const void *p, size_t size)
 /**
  * Write a password to the log file.
  *
- * @note Only actually does something if DEBUG_PASSWORD was defined during 
+ * @note Only actually does something if DEBUG_PASSWORD was defined during
  * compile-time.
  */
 _PUBLIC_ void dump_data_pw(const char *msg, const uint8_t * data, size_t len)
@@ -819,10 +830,21 @@ _PUBLIC_ void dump_data_pw(const char *msg, const uint8_t * data, size_t len)
 #endif
 }
 
+static void dump_data_addbuf_cb(const char *buf, void *private_data)
+{
+	char **str = private_data;
+	talloc_asprintf_addbuf(str, "%s", buf);
+}
+
+_PUBLIC_ void dump_data_addbuf(const uint8_t *buf, size_t buflen, char **str)
+{
+	dump_data_cb(buf, buflen, false, dump_data_addbuf_cb, str);
+}
+
 
 /**
  * see if a range of memory is all zero. A NULL pointer is considered
- * to be all zero 
+ * to be all zero
  */
 _PUBLIC_ bool all_zero(const uint8_t *ptr, size_t size)
 {
@@ -986,7 +1008,10 @@ _PUBLIC_ size_t strhex_to_str(char *p, size_t p_len, const char *strhex, size_t
 _PUBLIC_ DATA_BLOB strhex_to_data_blob(TALLOC_CTX *mem_ctx, const char *strhex)
 {
 	DATA_BLOB ret_blob = data_blob_talloc(mem_ctx, NULL, strlen(strhex)/2+1);
-
+	if (ret_blob.data == NULL) {
+		/* ret_blob.length is already 0 */
+		return ret_blob;
+	}
 	ret_blob.length = strhex_to_str((char *)ret_blob.data, ret_blob.length,
 					strhex,
 					strlen(strhex));
@@ -995,9 +1020,9 @@ _PUBLIC_ DATA_BLOB strhex_to_data_blob(TALLOC_CTX *mem_ctx, const char *strhex)
 }
 
 /**
- * Parse a hex dump and return a data blob. Hex dump is structured as 
+ * Parse a hex dump and return a data blob. Hex dump is structured as
  * is generated from dump_data_cb() elsewhere in this file
- * 
+ *
  */
 _PUBLIC_ DATA_BLOB hexdump_to_data_blob(TALLOC_CTX *mem_ctx, const char *hexdump, size_t hexdump_len)
 {
@@ -1011,10 +1036,10 @@ _PUBLIC_ DATA_BLOB hexdump_to_data_blob(TALLOC_CTX *mem_ctx, const char *hexdump
 	if (hexdump_len % 77) {
 		hexdump_byte_count += ((hexdump_len % 77) - 59 - 2);
 	}
-	
+
 	ret_blob = data_blob_talloc(mem_ctx, NULL, hexdump_byte_count+1);
 	for (; i+1 < hexdump_len && hexdump[i] != 0 && hexdump[i+1] != 0; i++) {
-		if ((i%77) == 0) 
+		if ((i%77) == 0)
 			i += 7; /* Skip the offset at the start of the line */
 		if ((i%77) < 56) { /* position 56 is after both hex chunks */
 			if (hexdump[i] != ' ') {
@@ -1030,7 +1055,7 @@ _PUBLIC_ DATA_BLOB hexdump_to_data_blob(TALLOC_CTX *mem_ctx, const char *hexdump
 		}
 	}
 	ret_blob.length = char_count;
-	
+
 	return ret_blob;
 }
 
@@ -1066,7 +1091,7 @@ _PUBLIC_ char *hex_encode_talloc(TALLOC_CTX *mem_ctx, const unsigned char *buff_
 }
 
 /**
-  varient of strcmp() that handles NULL ptrs
+  variant of strcmp() that handles NULL ptrs
 **/
 _PUBLIC_ int strcmp_safe(const char *s1, const char *s2)
 {
@@ -1097,6 +1122,14 @@ _PUBLIC_ size_t ascii_len_n(const char *src, size_t n)
 	return len;
 }
 
+_PUBLIC_ bool mem_equal_const_time(const void *s1, const void *s2, size_t n)
+{
+	/* Ensure we won't overflow the unsigned index used by gnutls. */
+	SMB_ASSERT(n <= UINT_MAX);
+
+	return gnutls_memcmp(s1, s2, n) == 0;
+}
+
 struct anonymous_shared_header {
 	union {
 		size_t length;
