@@ -43,7 +43,7 @@
 #include "lib/param/loadparm.h"
 #include "libcli/auth/netlogon_creds_cli.h"
 #include "auth/credentials/credentials.h"
-#include "krb5_env.h"
+#include "auth/gensec/gensec.h"
 #include "libsmb/dsgetdcname.h"
 #include "rpc_client/util_netlogon.h"
 #include "libnet/libnet_join_offline.h"
@@ -131,58 +131,35 @@ static void libnet_unjoin_set_error_string(TALLOC_CTX *mem_ctx,
 static ADS_STATUS libnet_connect_ads(const char *dns_domain_name,
 				     const char *netbios_domain_name,
 				     const char *dc_name,
-				     const char *user_name,
-				     const char *password,
-				     const char *ccname,
+				     struct cli_credentials *creds,
+				     TALLOC_CTX *mem_ctx,
 				     ADS_STRUCT **ads)
 {
+	TALLOC_CTX *tmp_ctx = talloc_stackframe();
 	ADS_STATUS status;
 	ADS_STRUCT *my_ads = NULL;
-	char *cp;
 
-	my_ads = ads_init(dns_domain_name,
+	my_ads = ads_init(tmp_ctx,
+			  dns_domain_name,
 			  netbios_domain_name,
 			  dc_name,
 			  ADS_SASL_SEAL);
 	if (!my_ads) {
-		return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+		status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+		goto out;
 	}
 
-	my_ads->auth.flags |= ADS_AUTH_ALLOW_NTLMSSP;
-
-	if (user_name) {
-		SAFE_FREE(my_ads->auth.user_name);
-		my_ads->auth.user_name = SMB_STRDUP(user_name);
-		if ((cp = strchr_m(my_ads->auth.user_name, '@'))!=0) {
-			*cp++ = '\0';
-			SAFE_FREE(my_ads->auth.realm);
-			my_ads->auth.realm = smb_xstrdup(cp);
-			if (!strupper_m(my_ads->auth.realm)) {
-				ads_destroy(&my_ads);
-				return ADS_ERROR_LDAP(LDAP_NO_MEMORY);
-			}
-		}
-	}
-
-	if (password) {
-		SAFE_FREE(my_ads->auth.password);
-		my_ads->auth.password = SMB_STRDUP(password);
-	}
-
-	if (ccname != NULL) {
-		SAFE_FREE(my_ads->auth.ccache_name);
-		my_ads->auth.ccache_name = SMB_STRDUP(ccname);
-		setenv(KRB5_ENV_CCNAME, my_ads->auth.ccache_name, 1);
-	}
-
-	status = ads_connect_user_creds(my_ads);
+	status = ads_connect_creds(my_ads, creds);
 	if (!ADS_ERR_OK(status)) {
-		ads_destroy(&my_ads);
-		return status;
+		goto out;
 	}
 
-	*ads = my_ads;
-	return ADS_SUCCESS;
+	*ads = talloc_move(mem_ctx, &my_ads);
+
+	status = ADS_SUCCESS;
+out:
+	TALLOC_FREE(tmp_ctx);
+	return status;
 }
 
 /****************************************************************
@@ -193,54 +170,48 @@ static ADS_STATUS libnet_join_connect_ads(TALLOC_CTX *mem_ctx,
 					  bool use_machine_creds)
 {
 	ADS_STATUS status;
-	const char *username;
-	const char *password;
-	const char *ccname = NULL;
+	struct cli_credentials *creds = NULL;
 
 	if (use_machine_creds) {
+		const char *username = NULL;
+		NTSTATUS ntstatus;
+
 		if (r->in.machine_name == NULL ||
 		    r->in.machine_password == NULL) {
 			return ADS_ERROR_NT(NT_STATUS_INVALID_PARAMETER);
 		}
-		username = talloc_asprintf(mem_ctx, "%s$",
-					   r->in.machine_name);
-		if (username == NULL) {
-			return ADS_ERROR(LDAP_NO_MEMORY);
+		if (r->out.dns_domain_name != NULL) {
+			username = talloc_asprintf(mem_ctx, "%s$@%s",
+						   r->in.machine_name,
+						   r->out.dns_domain_name);
+			if (username == NULL) {
+				return ADS_ERROR(LDAP_NO_MEMORY);
+			}
+		} else {
+			username = talloc_asprintf(mem_ctx, "%s$",
+						   r->in.machine_name);
+			if (username == NULL) {
+				return ADS_ERROR(LDAP_NO_MEMORY);
+			}
+		}
+
+		ntstatus = ads_simple_creds(mem_ctx,
+					    r->out.netbios_domain_name,
+					    username,
+					    r->in.machine_password,
+					    &creds);
+		if (!NT_STATUS_IS_OK(ntstatus)) {
+			return ADS_ERROR_NT(ntstatus);
 		}
-		password = r->in.machine_password;
-		ccname = "MEMORY:libnet_join_machine_creds";
 	} else {
-		char *p = NULL;
-
-		username = r->in.admin_account;
-
-		p = strchr(r->in.admin_account, '@');
-		if (p == NULL) {
-			username = talloc_asprintf(mem_ctx, "%s@%s",
-						   r->in.admin_account,
-						   r->in.admin_domain);
-		}
-		if (username == NULL) {
-			return ADS_ERROR(LDAP_NO_MEMORY);
-		}
-		password = r->in.admin_password;
-
-		/*
-		 * when r->in.use_kerberos is set to allow "net ads join -k" we
-		 * may not override the provided credential cache - gd
-		 */
-
-		if (!r->in.use_kerberos) {
-			ccname = "MEMORY:libnet_join_user_creds";
-		}
+		creds = r->in.admin_credentials;
 	}
 
 	status = libnet_connect_ads(r->out.dns_domain_name,
 				    r->out.netbios_domain_name,
 				    r->in.dc_name,
-				    username,
-				    password,
-				    ccname,
+				    creds,
+				    r,
 				    &r->in.ads);
 	if (!ADS_ERR_OK(status)) {
 		libnet_join_set_error_string(mem_ctx, r,
@@ -295,9 +266,8 @@ static ADS_STATUS libnet_unjoin_connect_ads(TALLOC_CTX *mem_ctx,
 	status = libnet_connect_ads(r->in.domain_name,
 				    r->in.domain_name,
 				    r->in.dc_name,
-				    r->in.admin_account,
-				    r->in.admin_password,
-				    NULL,
+				    r->in.admin_credentials,
+				    r,
 				    &r->in.ads);
 	if (!ADS_ERR_OK(status)) {
 		libnet_unjoin_set_error_string(mem_ctx, r,
@@ -522,12 +492,13 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 	char *spn = NULL;
 	const char **netbios_aliases = NULL;
 	const char **addl_hostnames = NULL;
+	const char *dns_hostname = NULL;
 
 	/* Find our DN */
 
 	status = libnet_join_find_machine_acct(mem_ctx, r);
 	if (!ADS_ERR_OK(status)) {
-		return status;
+		goto done;
 	}
 
 	status = libnet_join_get_machine_spns(frame,
@@ -578,6 +549,34 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 		goto done;
 	}
 
+	/*
+	 * Register dns_hostname if needed, add_uniq_spn() will avoid
+	 * duplicates.
+	 */
+	if (r->in.dnshostname != NULL) {
+		dns_hostname = talloc_strdup(frame, r->in.dnshostname);
+	} else {
+		dns_hostname = talloc_asprintf(frame,
+					       "%s.%s",
+					       r->in.machine_name,
+					       r->out.dns_domain_name);
+	}
+	if (dns_hostname == NULL) {
+		status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+		goto done;
+	}
+
+	spn = talloc_asprintf(frame, "HOST/%s", dns_hostname);
+	if (spn == NULL) {
+		status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+		goto done;
+	}
+
+	status = add_uniq_spn(frame, spn, &spn_array, &num_spns);
+	if (!ADS_ERR_OK(status)) {
+		goto done;
+	}
+
 	for (netbios_aliases = lp_netbios_aliases();
 	     netbios_aliases != NULL && *netbios_aliases != NULL;
 	     netbios_aliases++) {
@@ -605,6 +604,10 @@ static ADS_STATUS libnet_join_set_machine_spn(TALLOC_CTX *mem_ctx,
 		fstr_sprintf(my_alias, "%s.%s",
 			     *netbios_aliases,
 			     lp_dnsdomain());
+		if (!strlower_m(my_alias)) {
+			status = ADS_ERROR_LDAP(LDAP_NO_MEMORY);
+			goto done;
+		}
 
 		spn = talloc_asprintf(frame, "HOST/%s", my_alias);
 		if (spn == NULL) {
@@ -864,15 +867,9 @@ static ADS_STATUS libnet_join_set_etypes(TALLOC_CTX *mem_ctx,
 static bool libnet_join_create_keytab(TALLOC_CTX *mem_ctx,
 				      struct libnet_JoinCtx *r)
 {
-	if (!USE_SYSTEM_KEYTAB) {
-		return true;
-	}
+	NTSTATUS ntstatus = sync_pw2keytabs(r->in.dc_name);
 
-	if (ads_keytab_create_default(r->in.ads) != 0) {
-		return false;
-	}
-
-	return true;
+	return NT_STATUS_IS_OK(ntstatus);
 }
 
 /****************************************************************
@@ -904,12 +901,11 @@ static bool libnet_join_derive_salting_principal(TALLOC_CTX *mem_ctx,
 	}
 
 	salt = talloc_strdup(mem_ctx, std_salt);
+	SAFE_FREE(std_salt);
 	if (!salt) {
 		return false;
 	}
 
-	SAFE_FREE(std_salt);
-
 	/* if it's a Windows functional domain, we have to look for the UPN */
 
 	if (domain_func == DS_DOMAIN_FUNCTION_2000) {
@@ -1006,12 +1002,7 @@ static ADS_STATUS libnet_join_post_processing_ads_modify(TALLOC_CTX *mem_ctx,
 		 * to update msDS-SupportedEncryptionTypes reliable
 		 */
 
-		if (r->in.ads->auth.ccache_name != NULL) {
-			ads_kdestroy(r->in.ads->auth.ccache_name);
-			r->in.ads->auth.ccache_name = NULL;
-		}
-
-		ads_destroy(&r->in.ads);
+		TALLOC_FREE(r->in.ads);
 
 		status = libnet_join_connect_ads_machine(mem_ctx, r);
 		if (!ADS_ERR_OK(status)) {
@@ -1073,40 +1064,16 @@ static bool libnet_join_joindomain_store_secrets(TALLOC_CTX *mem_ctx,
  Connect dc's IPC$ share
 ****************************************************************/
 
-static NTSTATUS libnet_join_connect_dc_ipc(const char *dc,
-					   const char *user,
-					   const char *domain,
-					   const char *pass,
-					   bool use_kerberos,
+static NTSTATUS libnet_join_connect_dc_ipc(TALLOC_CTX *mem_ctx,
+					   const char *dc,
+					   struct cli_credentials *creds,
 					   struct cli_state **cli)
 {
-	TALLOC_CTX *frame = talloc_stackframe();
-	bool fallback_after_kerberos = false;
-	bool use_ccache = false;
-	bool pw_nt_hash = false;
-	struct cli_credentials *creds = NULL;
 	int flags = CLI_FULL_CONNECTION_IPC;
 	NTSTATUS status;
 
-	if (use_kerberos && pass) {
-		fallback_after_kerberos = true;
-	}
-
-	creds = cli_session_creds_init(frame,
-				       user,
-				       domain,
-				       NULL, /* realm (use default) */
-				       pass,
-				       use_kerberos,
-				       fallback_after_kerberos,
-				       use_ccache,
-				       pw_nt_hash);
-	if (creds == NULL) {
-		TALLOC_FREE(frame);
-		return NT_STATUS_NO_MEMORY;
-	}
-
-	status = cli_full_connection_creds(cli,
+	status = cli_full_connection_creds(mem_ctx,
+					   cli,
 					   NULL,
 					   dc,
 					   NULL, 0,
@@ -1114,11 +1081,9 @@ static NTSTATUS libnet_join_connect_dc_ipc(const char *dc,
 					   creds,
 					   flags);
 	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(frame);
 		return status;
 	}
 
-	TALLOC_FREE(frame);
 	return NT_STATUS_OK;
 }
 
@@ -1130,28 +1095,27 @@ static NTSTATUS libnet_join_lookup_dc_rpc(TALLOC_CTX *mem_ctx,
 					  struct libnet_JoinCtx *r,
 					  struct cli_state **cli)
 {
+	TALLOC_CTX *frame = talloc_stackframe();
 	struct rpc_pipe_client *pipe_hnd = NULL;
 	struct policy_handle lsa_pol;
 	NTSTATUS status, result;
 	union lsa_PolicyInformation *info = NULL;
+	struct cli_credentials *creds = NULL;
 	struct dcerpc_binding_handle *b;
-	const char *account = r->in.admin_account;
-	const char *domain = r->in.admin_domain;
-	const char *password = r->in.admin_password;
-	bool use_kerberos = r->in.use_kerberos;
 
 	if (r->in.join_flags & WKSSVC_JOIN_FLAGS_JOIN_UNSECURE) {
-		account = "";
-		domain = "";
-		password = NULL;
-		use_kerberos = false;
+		creds = cli_credentials_init_anon(frame);
+		if (creds == NULL) {
+			status = NT_STATUS_NO_MEMORY;
+			goto done;
+		}
+	} else {
+		creds = r->in.admin_credentials;
 	}
 
-	status = libnet_join_connect_dc_ipc(r->in.dc_name,
-					    account,
-					    domain,
-					    password,
-					    use_kerberos,
+	status = libnet_join_connect_dc_ipc(mem_ctx,
+					    r->in.dc_name,
+					    creds,
 					    cli);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
@@ -1211,6 +1175,7 @@ static NTSTATUS libnet_join_lookup_dc_rpc(TALLOC_CTX *mem_ctx,
 	TALLOC_FREE(pipe_hnd);
 
  done:
+	TALLOC_FREE(frame);
 	return status;
 }
 
@@ -1223,24 +1188,15 @@ static NTSTATUS libnet_join_joindomain_rpc_unsecure(TALLOC_CTX *mem_ctx,
 						    struct cli_state *cli)
 {
 	TALLOC_CTX *frame = talloc_stackframe();
-	struct rpc_pipe_client *authenticate_pipe = NULL;
 	struct rpc_pipe_client *passwordset_pipe = NULL;
 	struct cli_credentials *cli_creds;
 	struct netlogon_creds_cli_context *netlogon_creds = NULL;
-	struct netlogon_creds_CredentialState *creds = NULL;
-	uint32_t netlogon_flags = 0;
+	const struct sockaddr_storage *remote_sockaddr = NULL;
 	size_t len = 0;
 	bool ok;
 	DATA_BLOB new_trust_blob = data_blob_null;
 	NTSTATUS status;
 
-	status = cli_rpc_pipe_open_noauth(cli, &ndr_table_netlogon,
-					  &authenticate_pipe);
-	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(frame);
-		return status;
-	}
-
 	if (!r->in.machine_password) {
 		int security = r->in.ads ? SEC_ADS : SEC_DOMAIN;
 
@@ -1268,59 +1224,43 @@ static NTSTATUS libnet_join_joindomain_rpc_unsecure(TALLOC_CTX *mem_ctx,
 						r->in.secure_channel_type);
 
 	/* according to WKSSVC_JOIN_FLAGS_MACHINE_PWD_PASSED */
-	cli_credentials_set_password(cli_creds, r->in.admin_password,
+	cli_credentials_set_password(cli_creds,
+				     r->in.passed_machine_password,
 				     CRED_SPECIFIED);
 
-	status = rpccli_create_netlogon_creds_ctx(
-		cli_creds, authenticate_pipe->desthost, r->in.msg_ctx,
-		frame, &netlogon_creds);
+	cli_credentials_add_gensec_features(cli_creds,
+					    GENSEC_FEATURE_NO_DELEGATION,
+					    CRED_SPECIFIED);
+
+	remote_sockaddr = smbXcli_conn_remote_sockaddr(cli->conn);
+
+	status = rpccli_create_netlogon_creds_ctx(cli_creds,
+						  r->in.dc_name,
+						  r->in.msg_ctx,
+						  frame,
+						  &netlogon_creds);
 	if (!NT_STATUS_IS_OK(status)) {
 		TALLOC_FREE(frame);
 		return status;
 	}
 
-	status = rpccli_setup_netlogon_creds(
-		cli, NCACN_NP, netlogon_creds, true /* force_reauth */,
-		cli_creds);
+	status = rpccli_connect_netlogon(cli,
+					 NCACN_NP,
+					 r->in.dc_name,
+					 remote_sockaddr,
+					 netlogon_creds,
+					 true, /* force_reauth */
+					 cli_creds,
+					 &passwordset_pipe);
 	if (!NT_STATUS_IS_OK(status)) {
 		TALLOC_FREE(frame);
 		return status;
 	}
 
-	status = netlogon_creds_cli_get(netlogon_creds, frame, &creds);
-	if (!NT_STATUS_IS_OK(status)) {
-		TALLOC_FREE(frame);
-		return status;
-	}
-
-	netlogon_flags = creds->negotiate_flags;
-	TALLOC_FREE(creds);
-
-	if (netlogon_flags & NETLOGON_NEG_AUTHENTICATED_RPC) {
-		const char *remote_name = smbXcli_conn_remote_name(cli->conn);
-		const struct sockaddr_storage *remote_sockaddr =
-			smbXcli_conn_remote_sockaddr(cli->conn);
-
-		status = cli_rpc_pipe_open_schannel_with_creds(
-				cli,
-				&ndr_table_netlogon,
-				NCACN_NP,
-				netlogon_creds,
-				remote_name,
-				remote_sockaddr,
-				&passwordset_pipe);
-		if (!NT_STATUS_IS_OK(status)) {
-			TALLOC_FREE(frame);
-			return status;
-		}
-	} else {
-		passwordset_pipe = authenticate_pipe;
-	}
-
 	len = strlen(r->in.machine_password);
 	ok = convert_string_talloc(frame, CH_UNIX, CH_UTF16,
 				   r->in.machine_password, len,
-				   (void **)&new_trust_blob.data,
+				   &new_trust_blob.data,
 				   &new_trust_blob.length);
 	if (!ok) {
 		status = NT_STATUS_UNMAPPABLE_CHARACTER;
@@ -1404,7 +1344,8 @@ static NTSTATUS libnet_join_joindomain_rpc(TALLOC_CTX *mem_ctx,
 
 	b = pipe_hnd->binding_handle;
 
-	status = cli_get_session_key(mem_ctx, pipe_hnd, &session_key);
+	status = dcerpc_binding_handle_transport_session_key(
+				b, mem_ctx, &session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		DEBUG(0,("Error getting session_key of SAM pipe. Error was %s\n",
 			nt_errstr(status)));
@@ -1696,18 +1637,15 @@ error:
 NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 			const char *netbios_domain_name,
 			const char *dc_name,
-			const bool use_kerberos)
+			enum credentials_use_kerberos kerberos_state)
 {
 	TALLOC_CTX *frame = talloc_stackframe();
 	struct cli_state *cli = NULL;
 	struct rpc_pipe_client *netlogon_pipe = NULL;
 	struct cli_credentials *cli_creds = NULL;
 	struct netlogon_creds_cli_context *netlogon_creds = NULL;
-	struct netlogon_creds_CredentialState *creds = NULL;
-	uint32_t netlogon_flags = 0;
 	NTSTATUS status;
 	int flags = CLI_FULL_CONNECTION_IPC;
-	const char *remote_name = NULL;
 	const struct sockaddr_storage *remote_sockaddr = NULL;
 
 	if (!dc_name) {
@@ -1730,13 +1668,17 @@ NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 	/* we don't want any old password */
 	cli_credentials_set_old_password(cli_creds, NULL, CRED_SPECIFIED);
 
-	if (use_kerberos) {
-		cli_credentials_set_kerberos_state(cli_creds,
-						   CRED_USE_KERBEROS_REQUIRED,
-						   CRED_SPECIFIED);
-	}
+	cli_credentials_set_kerberos_state(cli_creds,
+					   kerberos_state,
+					   CRED_SPECIFIED);
 
-	status = cli_full_connection_creds(&cli, NULL,
+	cli_credentials_add_gensec_features(cli_creds,
+					    GENSEC_FEATURE_NO_DELEGATION,
+					    CRED_SPECIFIED);
+
+	status = cli_full_connection_creds(frame,
+					   &cli,
+					   NULL,
 					   dc_name,
 					   NULL, 0,
 					   "IPC$", "IPC",
@@ -1752,7 +1694,8 @@ NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 			return NT_STATUS_NO_MEMORY;
 		}
 
-		status = cli_full_connection_creds(&cli,
+		status = cli_full_connection_creds(frame,
+						   &cli,
 						   NULL,
 						   dc_name,
 						   NULL, 0,
@@ -1766,6 +1709,8 @@ NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 		return status;
 	}
 
+	remote_sockaddr = smbXcli_conn_remote_sockaddr(cli->conn);
+
 	status = rpccli_create_netlogon_creds_ctx(cli_creds,
 						  dc_name,
 						  msg_ctx,
@@ -1777,61 +1722,26 @@ NTSTATUS libnet_join_ok(struct messaging_context *msg_ctx,
 		return status;
 	}
 
-	status = rpccli_setup_netlogon_creds(cli, NCACN_NP,
-					     netlogon_creds,
-					     true, /* force_reauth */
-					     cli_creds);
+	status = rpccli_connect_netlogon(cli,
+					 NCACN_NP,
+					 dc_name,
+					 remote_sockaddr,
+					 netlogon_creds,
+					 true, /* force_reauth */
+					 cli_creds,
+					 &netlogon_pipe);
 	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(0,("connect_to_domain_password_server: "
-			 "unable to open the domain client session to "
-			 "machine %s. Flags[0x%08X] Error was : %s.\n",
-			 dc_name, (unsigned)netlogon_flags,
-			 nt_errstr(status)));
-		cli_shutdown(cli);
-		TALLOC_FREE(frame);
-		return status;
-	}
-
-	status = netlogon_creds_cli_get(netlogon_creds,
-					talloc_tos(),
-					&creds);
-	if (!NT_STATUS_IS_OK(status)) {
-		cli_shutdown(cli);
-		TALLOC_FREE(frame);
-		return status;
-	}
-	netlogon_flags = creds->negotiate_flags;
-	TALLOC_FREE(creds);
-
-	if (!(netlogon_flags & NETLOGON_NEG_AUTHENTICATED_RPC)) {
-		cli_shutdown(cli);
-		TALLOC_FREE(frame);
-		return NT_STATUS_OK;
-	}
-
-	remote_name = smbXcli_conn_remote_name(cli->conn);
-	remote_sockaddr = smbXcli_conn_remote_sockaddr(cli->conn);
-
-	status = cli_rpc_pipe_open_schannel_with_creds(
-		cli, &ndr_table_netlogon, NCACN_NP,
-		netlogon_creds,
-		remote_name,
-		remote_sockaddr,
-		&netlogon_pipe);
-
-	TALLOC_FREE(netlogon_pipe);
-
-	if (!NT_STATUS_IS_OK(status)) {
-		DEBUG(0,("libnet_join_ok: failed to open schannel session "
+		DBG_ERR("failed to open schannel session "
 			"on netlogon pipe to server %s for domain %s. "
 			"Error was %s\n",
-			remote_name,
-			netbios_domain_name, nt_errstr(status)));
+			dc_name, netbios_domain_name, nt_errstr(status));
 		cli_shutdown(cli);
 		TALLOC_FREE(frame);
 		return status;
 	}
 
+	TALLOC_FREE(netlogon_pipe);
+
 	cli_shutdown(cli);
 	TALLOC_FREE(frame);
 	return NT_STATUS_OK;
@@ -1844,11 +1754,17 @@ static WERROR libnet_join_post_verify(TALLOC_CTX *mem_ctx,
 				      struct libnet_JoinCtx *r)
 {
 	NTSTATUS status;
+	enum credentials_use_kerberos kerberos_state = CRED_USE_KERBEROS_DESIRED;
+
+	if (r->in.admin_credentials != NULL) {
+		kerberos_state = cli_credentials_get_kerberos_state(
+					r->in.admin_credentials);
+	}
 
 	status = libnet_join_ok(r->in.msg_ctx,
 				r->out.netbios_domain_name,
 				r->in.dc_name,
-				r->in.use_kerberos);
+				kerberos_state);
 	if (!NT_STATUS_IS_OK(status)) {
 		libnet_join_set_error_string(mem_ctx, r,
 			"failed to verify domain membership after joining: %s",
@@ -1893,11 +1809,9 @@ static NTSTATUS libnet_join_unjoindomain_rpc(TALLOC_CTX *mem_ctx,
 	ZERO_STRUCT(domain_pol);
 	ZERO_STRUCT(user_pol);
 
-	status = libnet_join_connect_dc_ipc(r->in.dc_name,
-					    r->in.admin_account,
-					    r->in.admin_domain,
-					    r->in.admin_password,
-					    r->in.use_kerberos,
+	status = libnet_join_connect_dc_ipc(mem_ctx,
+					    r->in.dc_name,
+					    r->in.admin_credentials,
 					    &cli);
 	if (!NT_STATUS_IS_OK(status)) {
 		goto done;
@@ -2311,25 +2225,12 @@ static WERROR libnet_join_pre_processing(TALLOC_CTX *mem_ctx,
 		return WERR_OK;
 	}
 
-	if (!r->in.admin_domain) {
-		char *admin_domain = NULL;
-		char *admin_account = NULL;
-		bool ok;
-
-		ok = split_domain_user(mem_ctx,
-				       r->in.admin_account,
-				       &admin_domain,
-				       &admin_account);
-		if (!ok) {
-			return WERR_NOT_ENOUGH_MEMORY;
-		}
-
-		if (admin_domain != NULL) {
-			r->in.admin_domain = admin_domain;
-		} else {
-			r->in.admin_domain = r->in.domain_name;
-		}
-		r->in.admin_account = admin_account;
+	if (r->in.provision_computer_account_only) {
+		/*
+		 * When in the "provision_computer_account_only" path we do not
+		 * need to have access to secrets.tdb at all - gd
+		 */
+		return WERR_OK;
 	}
 
 	if (!secrets_init()) {
@@ -2460,9 +2361,7 @@ static WERROR libnet_join_post_processing(TALLOC_CTX *mem_ctx,
 
 static int libnet_destroy_JoinCtx(struct libnet_JoinCtx *r)
 {
-	if (r->in.ads) {
-		ads_destroy(&r->in.ads);
-	}
+	TALLOC_FREE(r->in.ads);
 
 	return 0;
 }
@@ -2472,9 +2371,7 @@ static int libnet_destroy_JoinCtx(struct libnet_JoinCtx *r)
 
 static int libnet_destroy_UnjoinCtx(struct libnet_UnjoinCtx *r)
 {
-	if (r->in.ads) {
-		ads_destroy(&r->in.ads);
-	}
+	TALLOC_FREE(r->in.ads);
 
 	return 0;
 }
@@ -2494,20 +2391,15 @@ WERROR libnet_init_JoinCtx(TALLOC_CTX *mem_ctx,
 
 	talloc_set_destructor(ctx, libnet_destroy_JoinCtx);
 
-	ctx->in.machine_name = talloc_strdup(mem_ctx, lp_netbios_name());
+	ctx->in.machine_name = talloc_strdup(ctx, lp_netbios_name());
 	W_ERROR_HAVE_NO_MEMORY(ctx->in.machine_name);
 
 	ctx->in.secure_channel_type = SEC_CHAN_WKSTA;
 
-	ctx->in.desired_encryption_types = ENC_CRC32 |
-					   ENC_RSA_MD5 |
-					   ENC_RC4_HMAC_MD5;
-#ifdef HAVE_ENCTYPE_AES128_CTS_HMAC_SHA1_96
+	ctx->in.desired_encryption_types = 0;
+	ctx->in.desired_encryption_types |= ENC_RC4_HMAC_MD5;
 	ctx->in.desired_encryption_types |= ENC_HMAC_SHA1_96_AES128;
-#endif
-#ifdef HAVE_ENCTYPE_AES256_CTS_HMAC_SHA1_96
 	ctx->in.desired_encryption_types |= ENC_HMAC_SHA1_96_AES256;
-#endif
 
 	*r = ctx;
 
@@ -2529,7 +2421,7 @@ WERROR libnet_init_UnjoinCtx(TALLOC_CTX *mem_ctx,
 
 	talloc_set_destructor(ctx, libnet_destroy_UnjoinCtx);
 
-	ctx->in.machine_name = talloc_strdup(mem_ctx, lp_netbios_name());
+	ctx->in.machine_name = talloc_strdup(ctx, lp_netbios_name());
 	W_ERROR_HAVE_NO_MEMORY(ctx->in.machine_name);
 
 	*r = ctx;
@@ -2669,7 +2561,6 @@ static WERROR libnet_DomainJoin(TALLOC_CTX *mem_ctx,
 	ADS_STATUS ads_status;
 #endif /* HAVE_ADS */
 	const char *pre_connect_realm = NULL;
-	const char *numeric_dcip = NULL;
 	const char *sitename = NULL;
 	struct netr_DsRGetDCNameInfo *info;
 	const char *dc;
@@ -2678,8 +2569,7 @@ static WERROR libnet_DomainJoin(TALLOC_CTX *mem_ctx,
 	/* Before contacting a DC, we can securely know
 	 * the realm only if the user specifies it.
 	 */
-	if (r->in.use_kerberos &&
-	    r->in.domain_name_type == JoinDomNameTypeDNS) {
+	if (r->in.domain_name_type == JoinDomNameTypeDNS) {
 		pre_connect_realm = r->in.domain_name;
 	}
 
@@ -2731,7 +2621,6 @@ static WERROR libnet_DomainJoin(TALLOC_CTX *mem_ctx,
 		return WERR_NERR_DCNOTFOUND;
 	}
 
-	numeric_dcip = info->dc_address + 2;
 	sitename = info->dc_site_name;
 	/* info goes out of scope but the memory stays
 	   allocated on the talloc context */
@@ -2741,8 +2630,9 @@ static WERROR libnet_DomainJoin(TALLOC_CTX *mem_ctx,
 
 	if (pre_connect_realm != NULL) {
 		struct sockaddr_storage ss = {0};
+		const char *numeric_dcip = info->dc_address + 2;
 
-		if (numeric_dcip != NULL) {
+		if (numeric_dcip[0] == '\0') {
 			if (!interpret_string_addr(&ss, numeric_dcip,
 						   AI_NUMERICHOST)) {
 				DBG_ERR(
@@ -2760,7 +2650,7 @@ static WERROR libnet_DomainJoin(TALLOC_CTX *mem_ctx,
 		}
 
 		/* The domain parameter is only used as modifier
-		 * to krb5.conf file name. _JOIN_ is is not a valid
+		 * to krb5.conf file name. _JOIN_ is not a valid
 		 * NetBIOS name so it cannot clash with another domain
 		 * -- Uri.
 		 */
@@ -2962,10 +2852,8 @@ static WERROR libnet_join_rollback(TALLOC_CTX *mem_ctx,
 	u->in.debug		= r->in.debug;
 	u->in.dc_name		= r->in.dc_name;
 	u->in.domain_name	= r->in.domain_name;
-	u->in.admin_account	= r->in.admin_account;
-	u->in.admin_password	= r->in.admin_password;
+	u->in.admin_credentials	= r->in.admin_credentials;
 	u->in.modify_config	= r->in.modify_config;
-	u->in.use_kerberos	= r->in.use_kerberos;
 	u->in.unjoin_flags	= WKSSVC_JOIN_FLAGS_JOIN_TYPE |
 				  WKSSVC_JOIN_FLAGS_ACCOUNT_DELETE;
 
@@ -3171,27 +3059,6 @@ static WERROR libnet_unjoin_pre_processing(TALLOC_CTX *mem_ctx,
 		return WERR_NERR_SETUPDOMAINCONTROLLER;
 	}
 
-	if (!r->in.admin_domain) {
-		char *admin_domain = NULL;
-		char *admin_account = NULL;
-		bool ok;
-
-		ok = split_domain_user(mem_ctx,
-				       r->in.admin_account,
-				       &admin_domain,
-				       &admin_account);
-		if (!ok) {
-			return WERR_NOT_ENOUGH_MEMORY;
-		}
-
-		if (admin_domain != NULL) {
-			r->in.admin_domain = admin_domain;
-		} else {
-			r->in.admin_domain = r->in.domain_name;
-		}
-		r->in.admin_account = admin_account;
-	}
-
 	if (!secrets_init()) {
 		libnet_unjoin_set_error_string(mem_ctx, r,
 			"Unable to open secrets database");
