@@ -782,7 +782,7 @@ again:
 					entry, gidNumber, memctx);
 		}
 		if ( ! tmp) { /* wow very strange entry, how did it match ? */
-			DEBUG(5, ("Unprobable match on (%s), no uidNumber, "
+			DEBUG(5, ("Improbable match on (%s), no uidNumber, "
 				  "nor gidNumber returned\n", sidstr));
 			TALLOC_FREE(sidstr);
 			continue;
@@ -852,7 +852,7 @@ again:
 
 	ret = NT_STATUS_OK;
 
-	/* mark all unknwon/expired ones as unmapped */
+	/* mark all unknown/expired ones as unmapped */
 	for (i = 0; ids[i]; i++) {
 		if (ids[i]->status != ID_MAPPED)
 			ids[i]->status = ID_UNMAPPED;
@@ -936,14 +936,13 @@ again:
 		bidx = idx;
 		for (i = 0; (i < IDMAP_LDAP_MAX_IDS) && ids[idx]; i++, idx++) {
 			struct dom_sid_buf buf;
-			filter = talloc_asprintf_append_buffer(filter, "(%s=%s)",
-					LDAP_ATTRIBUTE_SID,
+			filter = talloc_asprintf_append_buffer(filter, "("LDAP_ATTRIBUTE_SID"=%s)",
 					dom_sid_str_buf(ids[idx]->sid, &buf));
 			CHECK_ALLOC_DONE(filter);
 		}
 		filter = talloc_asprintf_append_buffer(filter, "))");
 		CHECK_ALLOC_DONE(filter);
-		DEBUG(10, ("Filter: [%s]", filter));
+		DEBUG(10, ("Filter: [%s]\n", filter));
 	} else {
 		bidx = 0;
 		idx = 1;
@@ -1007,7 +1006,7 @@ again:
 		map = idmap_find_map_by_sid(&ids[bidx], &sid);
 		if (!map) {
 			DEBUG(2, ("WARNING: couldn't find entry sid (%s) "
-				  "in ids", sidstr));
+				  "in ids\n", sidstr));
 			TALLOC_FREE(sidstr);
 			continue;
 		}
