@@ -1,4 +1,4 @@
-/* 
+/*
    Unix SMB/CIFS implementation.
 
    Winbind rpc backend functions
@@ -226,8 +226,8 @@ static NTSTATUS msrpc_name_to_sid(struct winbindd_domain *domain,
 	struct dom_sid *sids = NULL;
 	enum lsa_SidType *types = NULL;
 	char *full_name = NULL;
-	const char *names[1];
-	const char **domains;
+	const char *names[1] = { NULL, };
+	const char **domains = NULL;
 	NTSTATUS name_map_status = NT_STATUS_UNSUCCESSFUL;
 	char *mapped_name = NULL;
 
@@ -270,13 +270,14 @@ static NTSTATUS msrpc_name_to_sid(struct winbindd_domain *domain,
 	/* Return rid and type if lookup successful */
 
 	if (pdom_name != NULL) {
-		const char *dom_name;
+		const char *dom_name = NULL;
 
-		dom_name = talloc_strdup(mem_ctx, domains[0]);
-		if (dom_name == NULL) {
-			return NT_STATUS_NO_MEMORY;
+		if (domains[0] != NULL) {
+			dom_name = talloc_strdup(mem_ctx, domains[0]);
+			if (dom_name == NULL) {
+				return NT_STATUS_NO_MEMORY;
+			}
 		}
-
 		*pdom_name = dom_name;
 	}
 
@@ -544,6 +545,60 @@ done:
 	return status;
 }
 
+/* lookup alias membership */
+static NTSTATUS msrpc_lookup_aliasmem(struct winbindd_domain *domain,
+				      TALLOC_CTX *mem_ctx,
+				      const struct dom_sid *alias_sid,
+				      enum lsa_SidType type,
+				      uint32_t *pnum_sids,
+				      struct dom_sid **sid_mem)
+{
+	struct rpc_pipe_client *samr_pipe = NULL;
+	struct policy_handle dom_pol;
+	struct dom_sid *alias_members = NULL;
+	struct dom_sid_buf buf;
+	uint32_t num_groups = 0;
+	TALLOC_CTX *tmp_ctx = talloc_stackframe();
+	NTSTATUS status;
+
+	D_INFO("Lookup alias members in domain=%s for sid=%s.\n",
+	       domain->name,
+	       dom_sid_str_buf(alias_sid, &buf));
+
+	*pnum_sids = 0;
+
+	if (!winbindd_can_contact_domain(domain)) {
+		D_DEBUG("No incoming trust for domain %s\n", domain->name);
+		status = NT_STATUS_OK;
+		goto done;
+	}
+
+	status = cm_connect_sam(domain, tmp_ctx, false, &samr_pipe, &dom_pol);
+	if (!NT_STATUS_IS_OK(status)) {
+		goto done;
+	}
+
+	status = rpc_lookup_aliasmem(tmp_ctx,
+				     samr_pipe,
+				     &dom_pol,
+				     &domain->sid,
+				     alias_sid,
+				     type,
+				     &num_groups,
+				     &alias_members);
+	if (!NT_STATUS_IS_OK(status)) {
+		goto done;
+	}
+
+	*pnum_sids = num_groups;
+	if (sid_mem) {
+		*sid_mem = talloc_move(mem_ctx, &alias_members);
+	}
+
+done:
+	talloc_free(tmp_ctx);
+	return status;
+}
 
 /* Lookup group membership given a rid.   */
 static NTSTATUS msrpc_lookup_groupmem(struct winbindd_domain *domain,
@@ -926,7 +981,7 @@ NTSTATUS winbindd_lookup_sids(TALLOC_CTX *mem_ctx,
 
 	b = cli->binding_handle;
 
-	if (cli->transport->transport == NCACN_IP_TCP) {
+	if (dcerpc_binding_handle_get_transport(b) == NCACN_IP_TCP) {
 		use_lookupsids3 = true;
 	}
 
@@ -995,6 +1050,12 @@ static NTSTATUS winbindd_lookup_names(TALLOC_CTX *mem_ctx,
 	enum lsa_LookupNamesLevel level = LSA_LOOKUP_NAMES_ALL;
 
  connect:
+	if (domains == NULL) {
+		*domains = NULL;
+	}
+	*sids = NULL;
+	*types = NULL;
+
 	status = cm_connect_lsat(domain, mem_ctx, &cli, &lsa_policy);
 	if (!NT_STATUS_IS_OK(status)) {
 		return status;
@@ -1002,7 +1063,7 @@ static NTSTATUS winbindd_lookup_names(TALLOC_CTX *mem_ctx,
 
 	b = cli->binding_handle;
 
-	if (cli->transport->transport == NCACN_IP_TCP) {
+	if (dcerpc_binding_handle_get_transport(b) == NCACN_IP_TCP) {
 		use_lookupnames4 = true;
 	}
 
@@ -1044,6 +1105,52 @@ static NTSTATUS winbindd_lookup_names(TALLOC_CTX *mem_ctx,
 		status = NT_STATUS_ACCESS_DENIED;
 	}
 
+	if (!NT_STATUS_IS_OK(status)) {
+		return status;
+	}
+
+	if (NT_STATUS_EQUAL(result, NT_STATUS_NONE_MAPPED)) {
+		if (num_names > 0) {
+			uint32_t i;
+
+			*sids = talloc_zero_array(mem_ctx, struct dom_sid, num_names);
+			if (*sids == NULL) {
+				return NT_STATUS_NO_MEMORY;
+			}
+
+			*types = talloc_zero_array(mem_ctx, enum lsa_SidType, num_names);
+			if (*types == NULL) {
+				return NT_STATUS_NO_MEMORY;
+			}
+
+			for (i = 0; i < num_names; i++) {
+				(*types)[i] = SID_NAME_UNKNOWN;
+			}
+
+			if (domains != NULL) {
+				*domains = talloc_zero_array(mem_ctx, const char *, num_names);
+				if (*domains == NULL) {
+					return NT_STATUS_NO_MEMORY;
+				}
+			}
+		}
+
+		result = NT_STATUS_OK;
+	} else if (NT_STATUS_EQUAL(result, NT_STATUS_SOME_NOT_MAPPED)) {
+		if (talloc_array_length(*sids) != num_names) {
+			return NT_STATUS_INVALID_NETWORK_RESPONSE;
+		}
+		if (talloc_array_length(*types) != num_names) {
+			return NT_STATUS_INVALID_NETWORK_RESPONSE;
+		}
+		if (domains != NULL) {
+			if (talloc_array_length(*domains) != num_names) {
+				return NT_STATUS_INVALID_NETWORK_RESPONSE;
+			}
+		}
+		result = NT_STATUS_OK;
+	}
+
 	if (any_nt_status_not_ok(status, result, &status)) {
 		return status;
 	}
@@ -1053,17 +1160,19 @@ static NTSTATUS winbindd_lookup_names(TALLOC_CTX *mem_ctx,
 
 /* the rpc backend methods are exposed via this structure */
 struct winbindd_methods msrpc_methods = {
-	False,
-	msrpc_query_user_list,
-	msrpc_enum_dom_groups,
-	msrpc_enum_local_groups,
-	msrpc_name_to_sid,
-	msrpc_sid_to_name,
-	msrpc_rids_to_names,
-	msrpc_lookup_usergroups,
-	msrpc_lookup_useraliases,
-	msrpc_lookup_groupmem,
-	msrpc_lockout_policy,
-	msrpc_password_policy,
-	msrpc_trusted_domains,
+	.consistent		= false,
+
+	.query_user_list	= msrpc_query_user_list,
+	.enum_dom_groups	= msrpc_enum_dom_groups,
+	.enum_local_groups	= msrpc_enum_local_groups,
+	.name_to_sid		= msrpc_name_to_sid,
+	.sid_to_name		= msrpc_sid_to_name,
+	.rids_to_names		= msrpc_rids_to_names,
+	.lookup_usergroups	= msrpc_lookup_usergroups,
+	.lookup_useraliases	= msrpc_lookup_useraliases,
+	.lookup_groupmem	= msrpc_lookup_groupmem,
+	.lookup_aliasmem	= msrpc_lookup_aliasmem,
+	.lockout_policy		= msrpc_lockout_policy,
+	.password_policy	= msrpc_password_policy,
+	.trusted_domains	= msrpc_trusted_domains,
 };
