@@ -17,7 +17,9 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include <Python.h>
+#define SOURCE4_LIBRPC_INTERNALS 1
+
+#include "lib/replace/system/python.h"
 #include "python/py3compat.h"
 #include "includes.h"
 #include "python/modules.h"
@@ -130,12 +132,30 @@ static PyObject *py_iface_transfer_syntax(PyObject *obj, void *closure)
 static PyObject *py_iface_session_key(PyObject *obj, void *closure)
 {
 	dcerpc_InterfaceObject *iface = (dcerpc_InterfaceObject *)obj;
-	DATA_BLOB session_key;
+	TALLOC_CTX *frame = talloc_stackframe();
+	DATA_BLOB session_key = { .length = 0, };
+	static PyObject *session_key_obj = NULL;
+	NTSTATUS status;
 
-	NTSTATUS status = dcerpc_fetch_session_key(iface->pipe, &session_key);
-	PyErr_NTSTATUS_IS_ERR_RAISE(status);
+	if (iface->binding_handle == NULL) {
+		PyErr_SetNTSTATUS(NT_STATUS_NO_USER_SESSION_KEY);
+		TALLOC_FREE(frame);
+		return NULL;
+	}
 
-	return PyBytes_FromStringAndSize((const char *)session_key.data, session_key.length);
+	status = dcerpc_binding_handle_transport_session_key(iface->binding_handle,
+							     frame,
+							     &session_key);
+	if (!NT_STATUS_IS_OK(status)) {
+		PyErr_SetNTSTATUS(status);
+		TALLOC_FREE(frame);
+		return NULL;
+	}
+
+	session_key_obj = PyBytes_FromStringAndSize((const char *)session_key.data,
+						     session_key.length);
+	TALLOC_FREE(frame);
+	return session_key_obj;
 }
 
 static PyObject *py_iface_user_session_key(PyObject *obj, void *closure)
@@ -143,30 +163,18 @@ static PyObject *py_iface_user_session_key(PyObject *obj, void *closure)
 	dcerpc_InterfaceObject *iface = (dcerpc_InterfaceObject *)obj;
 	TALLOC_CTX *mem_ctx;
 	NTSTATUS status;
-	struct gensec_security *security = NULL;
 	DATA_BLOB session_key = data_blob_null;
 	static PyObject *session_key_obj = NULL;
 
-	if (iface->pipe == NULL) {
+	if (iface->binding_handle == NULL) {
 		PyErr_SetNTSTATUS(NT_STATUS_NO_USER_SESSION_KEY);
 		return NULL;
 	}
-
-	if (iface->pipe->conn == NULL) {
-		PyErr_SetNTSTATUS(NT_STATUS_NO_USER_SESSION_KEY);
-		return NULL;
-	}
-
-	if (iface->pipe->conn->security_state.generic_state == NULL) {
-		PyErr_SetNTSTATUS(NT_STATUS_NO_USER_SESSION_KEY);
-		return NULL;
-	}
-
-	security = iface->pipe->conn->security_state.generic_state;
-
 	mem_ctx = talloc_new(NULL);
 
-	status = gensec_session_key(security, mem_ctx, &session_key);
+	status = dcerpc_binding_handle_auth_session_key(iface->binding_handle,
+							mem_ctx,
+							&session_key);
 	if (!NT_STATUS_IS_OK(status)) {
 		talloc_free(mem_ctx);
 		PyErr_SetNTSTATUS(status);
@@ -296,13 +304,34 @@ static PyObject *py_iface_transport_encrypted(PyObject *self)
 {
 	dcerpc_InterfaceObject *iface = (dcerpc_InterfaceObject *)self;
 
-	if (dcerpc_transport_encrypted(iface->pipe)) {
+	if (dcerpc_binding_handle_transport_encrypted(iface->binding_handle)) {
 		Py_RETURN_TRUE;
 	}
 
 	Py_RETURN_FALSE;
 }
 
+static PyObject *py_iface_auth_info(PyObject *self)
+{
+	dcerpc_InterfaceObject *iface = (dcerpc_InterfaceObject *)self;
+	enum dcerpc_AuthType auth_type = DCERPC_AUTH_TYPE_NONE;
+	enum dcerpc_AuthLevel auth_level = DCERPC_AUTH_LEVEL_NONE;
+	PyObject *result = Py_None;
+
+	dcerpc_binding_handle_auth_info(iface->binding_handle,
+					&auth_type,
+					&auth_level);
+
+	result = Py_BuildValue("(I,I)",
+			       (unsigned)auth_type,
+			       (unsigned)auth_level);
+	if (result == NULL) {
+		return NULL;
+	}
+
+	return result;
+}
+
 static PyMethodDef dcerpc_interface_methods[] = {
 	{ "request", PY_DISCARD_FUNC_SIG(PyCFunction, py_iface_request),
 		METH_VARARGS|METH_KEYWORDS,
@@ -311,6 +340,9 @@ static PyMethodDef dcerpc_interface_methods[] = {
 	{ "transport_encrypted", PY_DISCARD_FUNC_SIG(PyCFunction, py_iface_transport_encrypted),
 		METH_NOARGS,
 		"Check if the DCE transport is encrypted" },
+	{ "auth_info", PY_DISCARD_FUNC_SIG(PyCFunction, py_iface_auth_info),
+		METH_NOARGS,
+		"Returns tuple of DCERPC auth type and auth level" },
 	{ NULL, NULL, 0, NULL },
 };
 
@@ -492,8 +524,7 @@ static void py_dcerpc_ndr_pointer_dealloc(PyObject* self)
 	struct py_dcerpc_ndr_pointer *obj =
 		pytalloc_get_type(self, struct py_dcerpc_ndr_pointer);
 
-	Py_DECREF(obj->value);
-	obj->value = NULL;
+	Py_CLEAR(obj->value);
 
 	self->ob_type->tp_free(self);
 }
@@ -512,7 +543,7 @@ static int py_dcerpc_ndr_pointer_set_value(PyObject *self, PyObject *value, void
 	struct py_dcerpc_ndr_pointer *obj =
 		pytalloc_get_type(self, struct py_dcerpc_ndr_pointer);
 
-	Py_DECREF(obj->value);
+	Py_CLEAR(obj->value);
 	obj->value = value;
 	Py_INCREF(obj->value);
 	return 0;
