@@ -14,7 +14,8 @@ failed=0
 samba4bindir="$BINDIR"
 samba_tool="$samba4bindir/samba-tool"
 
-testit() {
+testit()
+{
 	name="$1"
 	shift
 	cmdline="$*"
@@ -25,7 +26,7 @@ testit() {
 		echo "success: $name"
 	else
 		echo "failure: $name"
-		failed=`expr $failed + 1`
+		failed=$(expr $failed + 1)
 	fi
 	return $status
 }
@@ -34,7 +35,7 @@ testit "Test login with --machine-pass without kerberos" $VALGRIND $smbclient -c
 
 testit "Test login with --machine-pass and kerberos" $VALGRIND $smbclient -c 'ls' $CONFIGURATION //$SERVER/tmp --machine-pass -k
 
-testit "time" $VALGRIND $PYTHON $samba_tool time $SERVER $CONFIGURATION  -W "$DOMAIN" -U"$USERNAME%$PASSWORD" $@
+testit "time" $VALGRIND $PYTHON $samba_tool time $SERVER $CONFIGURATION -W "$DOMAIN" -U"$USERNAME%$PASSWORD" "$@"
 
 testit "domain level.show" $VALGRIND $PYTHON $samba_tool domain level show
 
