@@ -2,7 +2,7 @@
 
 if [ $# -lt 1 ]; then
 	cat <<EOF
-Usage: dbcheck.sh PREFIX RELEASE
+Usage: dbcheck-oldrelease.sh PREFIX RELEASE
 EOF
 	exit 1
 fi
@@ -408,14 +408,10 @@ dbcheck_clean()
 }
 
 # This should 'fail', because it returns the number of modified records.
-# We don't need to run this against 4.1 releases
+# We need to run this against all versions without adprep 2016 (before Samba 4.19)
 dbcheck_acl_reset()
 {
-	if [ x$RELEASE = x"release-4-0-0" -o x$RELEASE = x"alpha13" ]; then
-		$PYTHON $BINDIR/samba-tool dbcheck --reset-well-known-acls --cross-ncs --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
-	else
-		return 1
-	fi
+	$PYTHON $BINDIR/samba-tool dbcheck --reset-well-known-acls --cross-ncs --fix --yes -H tdb://$PREFIX_ABS/${RELEASE}/private/sam.ldb "$@"
 }
 # But having fixed it all up, this should pass.
 # We don't need to run this against 4.1.0rc3
